package com.junmp.jyzb.controller;

import com.junmp.jyzb.api.bean.dto.EqsBriefDto;
import com.junmp.jyzb.api.bean.dto.InventoryDto;
import com.junmp.jyzb.api.bean.query.CabinetReq;
import com.junmp.jyzb.api.bean.query.InventoryReq;
import com.junmp.jyzb.api.bean.query.InventorySumReq;
import com.junmp.jyzb.api.bean.query.WarehouseReq;
import com.junmp.jyzb.api.bean.req.EquipmentBindReq;
import com.junmp.jyzb.entity.InventorySummary;
import com.junmp.jyzb.service.InventoryService;
import com.junmp.jyzb.service.InventorySummaryService;
import com.junmp.jyzb.service.MQProductService;
import com.junmp.jyzb.utils.ResponseResult;
import com.junmp.v2.common.bean.request.ValidationApi;
import com.junmp.v2.common.bean.response.ApiRes;
import com.junmp.v2.db.api.page.PageResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

@RestController
@Slf4j
@RequestMapping("/Inventory")
@Api(tags = "库存模块")
public class InventoryController {
    @Resource
    public InventoryService inventoryService;
    @Resource
    public InventorySummaryService inventorySummaryService;
    @Resource
    public MQProductService mqProductService;
    @GetMapping("/testMQ")
    @ApiOperation("根据组织机构查询库存信息")
    public ApiRes<Boolean> testMQ( ) {
        String aa="";
        return ApiRes.success(mqProductService.sendMessage());
    }
    //根据组织机构id/单警柜id/仓库id查询出库存的汇总信息
    @PostMapping("/GetEquipmentInfo")
    @ApiOperation("根据组织机构查询库存汇总信息")
    public ApiRes<PageResult<InventorySummary>> getEquipmentInfo(@RequestBody @Validated(ValidationApi.edit.class) InventorySumReq req) {
        return ApiRes.success(inventorySummaryService.getEquipmentInfo(req));
    }


    @PostMapping("/SetInventoryMsg")
    @ApiOperation("填充汇总表信息")
    public ResponseResult setInventoryMsg() {
        ResponseResult returnMsg = inventoryService.setInventoryMsg();
        return returnMsg;
    }


    //根据装备id查询出装备的详细信息
    @PostMapping("/GetDetail")
    @ApiOperation("根据装备id查询装备明细信息")
    public ApiRes<InventoryDto> getDetail(@RequestBody InventoryReq req) {
        return  ApiRes.success(inventoryService.getInventoryDetail(req));
    }
    @PostMapping("/getEquipmentInfo")
    @ApiOperation("查询库存明细信息")
    public ResponseResult getEquipmentInfo(@RequestBody Map<String, Object> msg) {
        ResponseResult returnMsg = inventoryService.getEquipmentInfo(msg);
        return  returnMsg;
    }

    @PostMapping("/GetInventoryDevelop")
    @ApiOperation("获取装备明细表所有信息")
    public ResponseResult getInventoryDevelop() {
        ResponseResult returnMsg = inventoryService.getInventoryDevelop();
        return returnMsg;
    }

    @PostMapping("/InvertoryBind")
    @ApiOperation("装备绑定箱门")
    public ApiRes<Boolean> InvertoryBind(@RequestBody @Validated(ValidationApi.edit.class) EquipmentBindReq req){
        return ApiRes.success(inventoryService.InvertoryBind(req));
    }

    @PostMapping("/UpdateWarehouseInsNum")
    @ApiOperation("手动重置仓库库存数量信息")
    public ApiRes<Boolean> UpdateWarehouseInsNum(@RequestBody @Validated(ValidationApi.detail.class) WarehouseReq req){
        return ApiRes.success(inventoryService.UpdateWarehouseInsNum(req));
    }

    @PostMapping("/UpdateCabinetInsNum")
    @ApiOperation("手动重置单警柜库存数量信息")
    public ApiRes<Boolean> UpdateCabinetInsNum(@RequestBody @Validated(ValidationApi.detail.class)CabinetReq req){
        return ApiRes.success(inventoryService.UpdateCabinetInsNum(req));
    }

    @PostMapping("/GetInvInfoByEpc")
    @ApiOperation("根据EPC列表获取装备信息")
    public ApiRes<List<EqsBriefDto>> GetInvInfoByEpc(@RequestBody InventoryReq req){
        return ApiRes.success(inventoryService.GetInvInfoByEpc(req));
    }

    @PostMapping("/GetInventoryByOrgId")
    @ApiOperation("查询组织机构拥有装备")
    public ApiRes<List<EqsBriefDto>> getInventoryByOrgId(@RequestBody @Validated(ValidationApi.detail.class) InventoryReq req){
        return ApiRes.success(inventoryService.getInventoryByOrgId(req));
    }

    //根据组织机构/单警柜id/仓库id查询每件装备的详细信息
    @PostMapping("/EqsBySearchType")
    @ApiOperation("根据传入信息查询装备信息")
    public ApiRes<List<InventoryDto>> EqsBySearchType(@RequestBody @Validated(ValidationApi.edit.class) InventoryReq req ){
        return ApiRes.success(inventoryService.EqsBySearchType(req));
    }


}
