package com.junmp.jyzb.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.junmp.jyzb.api.bean.dto.EqsBriefDto;
import com.junmp.jyzb.api.bean.dto.InventoryDto;
import com.junmp.jyzb.api.bean.query.CabinetReq;
import com.junmp.jyzb.api.bean.query.InventoryReq;
import com.junmp.jyzb.api.bean.query.WarehouseReq;
import com.junmp.jyzb.api.bean.req.EquipmentBindReq;
import com.junmp.jyzb.api.exception.enums.CabinetBoxExceptionEnum;
import com.junmp.jyzb.api.exception.enums.CabinetExceptionEnum;
import com.junmp.jyzb.api.exception.enums.InventoryExceptionEnum;
import com.junmp.jyzb.entity.Cabinet;
import com.junmp.jyzb.entity.CabinetBox;
import com.junmp.jyzb.entity.Inventory;

import com.junmp.jyzb.entity.Warehouse;
import com.junmp.jyzb.service.*;
import com.junmp.jyzb.utils.HttpStatus;
import com.junmp.jyzb.utils.ResponseResult;
import com.junmp.jyzb.utils.ReturnMsg;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import com.junmp.jyzb.mapper.InventoryMapper;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

import static com.junmp.jyzb.utils.CheckBlank.checkNotBlank;

@Service
@Slf4j
public class InventoryServiceImpl extends ServiceImpl<InventoryMapper, Inventory> implements InventoryService {

    @Resource
    private InventoryMapper inventoryMapper;
    @Resource
    private EquipmentTypeService equipmentTypeService;
    @Resource
    private EquipmentSizeService equipmentSizeService;
    @Resource
    private WarehouseService warehouseService;
    @Resource
    private PubOrgService PubOrgService;

    @Resource
    private CabinetBoxService cabinetBoxService;

    @Resource
    private PubOrgService pubOrgService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private SupplierService supplierService;
    @Override
    public InventoryDto getInventoryDetail(InventoryReq req) {

        Inventory invMap=this.getById(req.getId());
        if (invMap == null) {
            return null;
        }
        InventoryDto bean = BeanUtil.toBean(invMap, InventoryDto.class);
        // 查询与库存表关联的其他数据
        String orgName= PubOrgService.getById(invMap.getOrgId()).getOrgName();
        String TypeName = equipmentTypeService.getById(invMap.getTypeId()).getName();
        String SizeName = equipmentSizeService.getById(invMap.getSizeId()).getName();
        // 创建并填充对象
        bean.setSizeName(SizeName);
        bean.setTypeName(TypeName);
        bean.setOrgName(orgName);

        // 填充其他关联信息的字段和对象引用
        return bean;
    }

    @Override
    public ResponseResult getEquipmentInfo(Map<String, Object> inMsg) {
        //参数校验
        try {
            checkNotBlank(inMsg.get("queryType"), "queryType不能为空");
            checkNotBlank(inMsg.get("typeId"), "typeId不能为空");
        } catch (IllegalArgumentException e) {
            return new ResponseResult(HttpStatus.ERROR, ReturnMsg.ERROR,e.getMessage());
        }

        //解析json中的信息
        String queryType = inMsg.get("queryType").toString();
        String idMsg = inMsg.get("typeId").toString();

        //根据传入的queryType来辨别要查询什么信息
        if (queryType.equals("org") || queryType.equals("warehouse") || queryType.equals("cabinet")){
            Map<String,Object> returnMsg = getLocationEquipmentInfo(queryType,idMsg);
            return new ResponseResult(HttpStatus.SUCCESS, ReturnMsg.PASS,returnMsg);
        }else {
            return new ResponseResult(HttpStatus.ERROR, ReturnMsg.ERROR,"输入的queryType信息有误");
        }

    }

    private Map<String, Object> getLocationEquipmentInfo(String queryType, String idMsg) {
        List<Map<String, Object>> allMsg = inventoryMapper.getLocationInventory(queryType,idMsg);

        Map<String, Object> resultMap = new HashMap<>();
        for (Map<String, Object> msg : allMsg) {
            String locationId = msg.get("locationId").toString();
            String locationName = msg.get("locationName").toString();
            String typeId = msg.get("typeId").toString();
            String typeName = msg.get("typeName").toString();
            String sizeId = msg.get("sizeId").toString();
            String sizeName = msg.get("sizeName").toString();
            int number = (int) msg.get("number");
            BigDecimal priceBigDecimal = (BigDecimal) msg.get("price");
            float price = priceBigDecimal.floatValue();

            //检查resultMap中是否存在orgId
            if (resultMap.containsKey(locationId)) {
                // 获取现有组织级数�
                Map<String, Object> orgData = (Map<String, Object>) resultMap.get(locationId);
                List<Map<String, Object>> orgTypeList = (List<Map<String, Object>>) orgData.get("typeList");

                // 检查组织类型列表中是否存在typeId
                boolean typeExist = false;
                for (Map<String, Object> typeData : orgTypeList) {
                    if (typeId.equals(typeData.get("typeId"))) {
                        // 获取现有的类型级别数�
                        List<Map<String, Object>> sizeList = (List<Map<String, Object>>) typeData.get("sizeList");
                        sizeList.add(createSizeData(sizeId,sizeName, number,price)); //向sizeList中记录number的信�
                        int typeNumber = (int) typeData.get("typeNum");
                        float typePrice = (float) typeData.get("typeTotalPrice");
                        typeData.put("typeNum", typeNumber + number); //计算装备数量
                        typeData.put("typeTotalPrice", typePrice + price);
                        typeExist = true;
                        break;
                    }
                }

                // 如果 orgTypeList 中不包含 typeId，则创建新的类型级别数据
                if (!typeExist) {
                    Map<String, Object> typeData = new HashMap<>();
                    typeData.put("typeId", typeId);
                    typeData.put("typeName", typeName);
                    typeData.put("typeNum", number);
                    typeData.put("typeTotalPrice", price);
                    List<Map<String, Object>> sizeList = new ArrayList<>();
                    sizeList.add(createSizeData(sizeId,sizeName, number,price));
                    typeData.put("sizeList", sizeList);
                    orgTypeList.add(typeData);
                }

                // 更新组织级别数据
                int orgNumber = (int) orgData.get("locationNum");
                float orgPrice = (float) orgData.get("totalPrice");
                orgData.put("locationNum", orgNumber + number);
                orgData.put("totalPrice", orgPrice + price);
            } else {
                // 创建新的组织级别数据
                Map<String, Object> orgData = new HashMap<>();
                orgData.put("locationType", queryType);
                orgData.put("locationId", locationId);
                orgData.put("locationName", locationName);
                orgData.put("locationNum", number);
                orgData.put("totalPrice", price);
                List<Map<String, Object>> orgTypeList = new ArrayList<>();
                Map<String, Object> typeData = new HashMap<>();
                typeData.put("typeId", typeId);
                typeData.put("typeName", typeName);
                typeData.put("typeNum", number);
                typeData.put("typeTotalPrice", price);
                List<Map<String, Object>> sizeList = new ArrayList<>();
                sizeList.add(createSizeData(sizeId,sizeName, number,price));
                typeData.put("sizeList", sizeList);
                orgTypeList.add(typeData);
                orgData.put("typeList", orgTypeList);
                resultMap.put(locationId, orgData);
            }
        }
        Map<String,Object> returnMsg = (Map<String, Object>) resultMap.get(idMsg);
        return returnMsg;
    }

    private static Map<String, Object> createSizeData(String sizeId,String sizeName, int number,float price) {
        Map<String, Object> sizeData = new HashMap<>();
        sizeData.put("sizeId", sizeId);
        sizeData.put("sizeName", sizeName);
        sizeData.put("sizeNum", number);
        sizeData.put("sizeTotalPrice", price);
        return sizeData;
    }

    @Override
    public ResponseResult setInventoryMsg(){
        inventoryMapper.deleteAllMsg(); //清空数据库中已存在的信息

        inventoryMapper.setOrgInventory(); //放入组织机构信息汇总

        inventoryMapper.setCabinetInventory(); //放入单警柜信息汇总

        inventoryMapper.setWarehouseInventory(); //放入仓库信息汇总
        return new ResponseResult(HttpStatus.SUCCESS, ReturnMsg.PASS);
    }



    @Override
    public List<EqsBriefDto> getInventoryByOrgId(InventoryReq req) {
       //判断组织机构是否存在
        pubOrgService.PubOrgExist(req.getOrgId());
        List<EqsBriefDto> eqsBriefDtoList =new ArrayList<>();
        List<Inventory> list = list(new LambdaQueryWrapper<Inventory>()
                .eq(Inventory::getOrgId, req.getOrgId()));
        for (Inventory inventory:list) {
            EqsBriefDto eqsBriefDto = new EqsBriefDto();
            BeanPlusUtil.copyProperties(inventory, eqsBriefDto);
            //设置型号名称，类别名称，供应商名称
            String sizeName = equipmentSizeService.getById(inventory.getSizeId()).getName();
            String typeName = equipmentTypeService.getById(inventory.getTypeId()).getName();
            String supplierName = supplierService.getById(inventory.getSupplierId()).getName();
            eqsBriefDto.setSizeName(sizeName);
            eqsBriefDto.setTypeName(typeName);
            eqsBriefDto.setSupplierName(supplierName);
            eqsBriefDtoList.add(eqsBriefDto);
        }
        return eqsBriefDtoList;
    }

    @Override
    public ResponseResult getInventoryDevelop() {
        return null;
    }


    //设置仓库库存数量（总数，在库数，出库数，价格）
    @Override
    public boolean UpdateWarehouseInsNum(WarehouseReq req) {
        Warehouse warehouse = warehouseService.WarehoustExist(req.getId());
        boolean b = warehouseService.SetInventoryInfo(warehouse.getId());
        return b;
    }

    //设置单警柜库存数量（总数，在库数，出库数，价格）
    @Override
    public boolean UpdateCabinetInsNum(CabinetReq req) {
        Cabinet cabinet = cabinetService.CabinetExist(req.getId());
        boolean b = cabinetService.SetInventoryInfo(cabinet.getId());
        return b;
    }

    //装备绑定箱门
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean InvertoryBind(EquipmentBindReq req) {
        //判断箱门是否存在
        CabinetBox one = cabinetBoxService.getOne(new LambdaQueryWrapper<CabinetBox>()
                .eq(CabinetBox::getCabinetId, req.getCabinetId())
                .eq(CabinetBox::getId, req.getCabinetBoxId()));
        if (Objects.isNull(one)){
            throw new ServiceException(CabinetBoxExceptionEnum.CABINETBOX_ISNOT_EXISTS);
        }
        //判断组织机构是否存在
        pubOrgService.PubOrgExist(req.getOrgId());
        List<InventoryReq> equipmentList = req.getEquipmentList();
        if (equipmentList.size()==0){
            throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
        }
        List<Inventory> inventoryList=new ArrayList<>();
        for (InventoryReq inventoryReq: equipmentList) {
            Inventory inventory = new Inventory();
            BeanPlusUtil.copyProperties(inventoryReq,inventory);
            inventory.setShelfId(req.getCabinetBoxId());
            inventoryList.add(inventory);
        }
        return updateBatchById(inventoryList);
    }

    //根据Epc查询装备信息
    @Override
    public List<EqsBriefDto> GetInvInfoByEpc(InventoryReq req) {
        List<EqsBriefDto> eqsBriefDtoList =new ArrayList<>();
        //判断传入的是否为空
        List<String> epcList = req.getEpcList();
        if (epcList.size()==0){
            throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
        }
        //迭代器遍历列表
        Iterator<String> iterator = epcList.iterator();
        while (iterator.hasNext()){
            String epc = iterator.next();
            Inventory inventory = getOne(new LambdaQueryWrapper<Inventory>().eq(Inventory::getEpc, epc));
            if (Objects.isNull(inventory)){
                throw new ServiceException(InventoryExceptionEnum.INVENTORY_NOT_EXIST);
            }
            EqsBriefDto eqsBriefDto = new EqsBriefDto();
            BeanPlusUtil.copyProperties(inventory, eqsBriefDto);
            //设置型号名称，类别名称，供应商名称
            String sizeName = equipmentSizeService.getById(inventory.getSizeId()).getName();
            String typeName = equipmentTypeService.getById(inventory.getTypeId()).getName();
            String supplierName = supplierService.getById(inventory.getSupplierId()).getName();
            eqsBriefDto.setSizeName(sizeName);
            eqsBriefDto.setTypeName(typeName);
            eqsBriefDto.setSupplierName(supplierName);
            eqsBriefDtoList.add(eqsBriefDto);
        }
        return eqsBriefDtoList;
    }

    @Override
    public List<InventoryDto> EqsBySearchType(InventoryReq req) {
        //判断传入的queryType参数是否是正确的
        if (!req.getQueryType().equals("org") && !req.getQueryType().equals("warehouse") && !req.getQueryType().equals("cabinet")){
            throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
        }
        //判断如果传递了类别，判断传递的值是否为空或者是否不存在
        if (ObjectUtil.isNotNull(req.getOrgId())){
            pubOrgService.PubOrgExist(req.getOrgId());
        }
        if (ObjectUtil.isNotNull(req.getWarehouseId())){
            warehouseService.WarehoustExist(req.getWarehouseId());
        }
        if (ObjectUtil.isNotNull(req.getCabinetId())){
            cabinetService.CabinetExist(req.getCabinetId());
        }
        List<InventoryDto> inventoryDtos = inventoryMapper.EqsBySearchType(req);
        return inventoryDtos;
    }


    /**
     * @Author: Zhaojw
     * @Description: 生成lambda表达式
     * @DateTime: 2023/7/20 13:08
     * @Params:
     * @Return
     */
    private LambdaQueryWrapper<Inventory> createWrapper(InventoryReq req) {
        LambdaQueryWrapper<Inventory> wrapper = new LambdaQueryWrapper<>();
        if (ObjectUtil.isEmpty(req)) {
            return wrapper;
        }
        //根据库存ID查询
        wrapper.eq(ObjectUtil.isNotEmpty(req.getId()), Inventory::getId, req.getId());
        //根据epc模糊查询
        wrapper.like(ObjectUtil.isNotEmpty(req.getEpc()), Inventory::getEpc, req.getEpc());
        //根据位置
        wrapper.eq(ObjectUtil.isNotEmpty(req.getLocationId()), Inventory::getLocationId, req.getLocationId());
        wrapper.orderByDesc(Inventory::getUpdateTime);
        return wrapper;
    }
}