package com.junmp.jyzb.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.junmp.jyzb.api.bean.req.UpdateEquipmentSizeReq;
import com.junmp.jyzb.api.bean.req.UpdateOrderReq;
import com.junmp.jyzb.api.exception.enums.OrderExceptionEnum;
import com.junmp.jyzb.api.exception.enums.ProductExceptionEnum;
import com.junmp.jyzb.entity.*;
import com.junmp.jyzb.mapper.OrderMainMapper;
import com.junmp.jyzb.service.OrderDetailService;
import com.junmp.jyzb.service.OrderService;
import com.junmp.jyzb.utils.DateTimeUtil;
import com.junmp.jyzb.utils.HttpStatus;
import com.junmp.jyzb.utils.ResponseResult;
import com.junmp.jyzb.utils.ReturnMsg;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

import static com.junmp.jyzb.utils.CheckBlank.checkNotBlank;

@Service
public class OrderServiceImpl  extends ServiceImpl<OrderMainMapper, OrderMain> implements OrderService {

    @Resource
    private OrderDetailService orderDetailService;

    @Resource
    private OrderMainMapper orderMapper;
    @Transactional
    @Override
    public String AddOrder(UpdateOrderReq req) {

        OrderMain order = new OrderMain();
        String orderId=UUID.randomUUID().toString();
        BeanPlusUtil.copyProperties(req, order);
        order.setId(orderId);
        List<OrderDetail> detailList = new ArrayList<>();
        req.getDetailList().forEach(p->
        {
            OrderDetail detail=new OrderDetail();
            BeanPlusUtil.copyProperties(p, detail);
            detail.setOrderId(orderId);
            detailList.add(detail);
        });
        orderDetailService.saveBatch(detailList);
        this.save(order);
        return  orderId;
    }

    @Override
    public ResponseResult showOrder(Map<String, Object> msg) {
        try {
            checkNotBlank(msg.get("orgId"), "orgId不能为空");
        } catch (IllegalArgumentException e) {
            return new ResponseResult(HttpStatus.ERROR, ReturnMsg.ERROR,e.getMessage());
        }

        String orgId = msg.get("orgId").toString();
        List<Map<String,Object>> allMsg = orderMapper.showOrder(orgId);

        return new ResponseResult(HttpStatus.SUCCESS, ReturnMsg.PASS,allMsg);
    }
    @Transactional
    @Override
    public Boolean updateOrder(UpdateOrderReq req) {
        OrderMain order = this.query(req);
        if (!order.getExamineState().equals("none"))
        {
            throw new ServiceException(OrderExceptionEnum.ORDER_CAN_NOT_UPDATE);
        }
        List<OrderDetail> updateList= orderDetailService.QueryList(order.getId());
        List<String> updateCheckList=new ArrayList<>();
        List<OrderDetail> addList = new ArrayList<>();
        req.getDetailList().forEach(detail -> {
            updateCheckList.add(detail.getId()); // 将id转换为String并添加到updateList中
        });
        boolean checkResult = this.hasCommonElements(updateCheckList,req.getDeleteList());


        if (req.getDeleteList().size()>0)
        {
            orderDetailService.removeBatchByIds(req.getDeleteList());
        }
        req.getDetailList().forEach(p->
        {
            if (p.getId()!=null)//修改
            {
                updateList.forEach(detail->
                {
                    if (detail.getId().equals(p.getId()))
                    {
                        BeanPlusUtil.copyProperties(p, detail);
                    }
                });

            }
            else//新增
            {
                OrderDetail detail=new OrderDetail();
                BeanPlusUtil.copyProperties(p, detail);
                detail.setOrderId(order.getId());
                addList.add(detail);
            }

        });
        orderDetailService.updateBatchById(updateList);
        orderDetailService.saveBatch(addList);
        this.save(order);
        return  true;
    }
    public boolean hasCommonElements(List<?> list1, List<?> list2) {
        for (Object item1 : list1) {
            if (list2.contains(item1)) {
                return true; // 找到相同值
            }
        }
        return false; // 没有相同值
    }

    @Override
    public ResponseResult getOrderDetail(Map<String, Object> msg) {
        try {
            checkNotBlank(msg.get("id"), "id不能为空");
        } catch (IllegalArgumentException e) {
            return new ResponseResult(HttpStatus.ERROR, ReturnMsg.ERROR,e.getMessage());
        }

        String id = msg.get("id").toString();
        Map<String,Object> oneMsg = orderMapper.getOrderDetail(id);

        return new ResponseResult(HttpStatus.SUCCESS, ReturnMsg.PASS,oneMsg);
    }

    private OrderMain query(UpdateOrderReq req) {
        OrderMain order = this.getById(req.getId());

        if (ObjectUtil.isNull(order)) {
            throw new ServiceException(OrderExceptionEnum.ORDER_NOT_EXIST);
        }
        return order;
    }

}
